;========================================================================
;   BASIC ATOM PRO 28-M  microcontroller
; CaveMap  
; Version         Date            Description
;   0.2         4th Jan 2007     First test
;
; Program that reads 2 RS-232 instruments and display result in LCD
; Save result on MMC card
; Jose sheim Ojeda, eLab University of Bergen
;========================================================================

; VARIABLES
index         Var Word ;measurement index (number) 0..65535  16bits
strIndex      Var Byte(3)
filename      Var Byte(9) ; form like f001.txt + null terminated
filenr        Var Byte
filepos       Var Word
filesize      Var Word
bytesToWrite  Var Word
menumode      Var Byte
k             Var Byte
n             Var Word
m             Var Byte
l             Var Byte
dummy         Var Byte
waitcount     Var Long
saveData      Var Byte(512)
InData        Var Byte(20)
strHeading    Var Byte(5)
strTilt       Var Byte(5) ; in degrees 
strDistance   Var Byte(6) ; in mm , maks is 999 meter (cutting of last digit 0.1mm)
inCompass     Var Byte(50)
inLaser       Var Byte(15)
button0       Var Byte ; check if button0 pressed 0 or 1 (true)
button1       Var Byte ; check if button1 pressed 0 or 1 (true)
debounce0     Var Byte ;var to prevent button0 debouncing
debounce1     Var Byte ;var to prevent button1 debouncing
SD_ok         Var Byte ; reply from SD card if commando ok
compass_ok    Var Byte ; tells if possible to read compass  (possible = 1)
laser_ok      Var Byte ;                           laser
bDeleteFile   Var Byte ; if 1 yes delete data file, if = 0 no
bLaserOn      Var Byte ; check laser spot status =1 laser is on, if =0 laser is off 
offset        Var long; offset for distance measurement in mm 
lDistance     Var long ; numerical value of strDistance 
bChgFile      Var Byte ; if change filename is true
ADinput       Var Word ; input from reading 10 bits adc
BatteryVolt   Var Float; Measure of battery volt

; CONSTANTS

;LCD display pins
DB4           Con P8  
DB5           Con P9
DB6           Con P10
DB7           Con P11 
RegSel        Con P12 ; RS
Clk           Con P13 ; enable E#
RdWrPin       Con P6 ;dummy


;SDA           Con P6
;SCL           Con P7
;Serial communication using 3 software serialports
; 1 and 2 are converted to RS232 level, 3 is TTL level
TX1           Con P2 ; Laser to Com1
RX1           Con P3
TX2           Con P4 ; Compass to Com2
RX2           Con P5
TX3           Con P15 ; SD memory to Com3
RX3           Con P14

PwrOff        Con P7

CR            Con 13 ; carriage return
LF            Con 10 ; line feed
TAB           Con 9  ; horisontal tabulator
SPACE_        Con 32
True          Con 1
False         Con 0


; -- Program starts here --
output PwrOff ; PowerSave pin output and active high
low PwrOff
input P18
InData = rep "X"\20 ; loads 20 X char into InData
; Mechanical pushbutton
input P0 ; Button0 input and active low
input P1 ; Button1 input and active low



; R2 and T2 to compass
input RX2
output TX2
; Com for Laser (breadbord)
;R1 pin2 breadboard til pin13 MAX232 TTL pin 12, rd til P14
input RX1
;T1 TTL pin11 converter pin 14 og ut p pin3 breadboard
output TX1

'listen to the -reset output from the watchdog, pin 14 on AtomPro 28-M (only on this)
input P18
'ax0, hardware pin 13 on AtomPro 28-M (only on this), also call P19 when used as ttl I/O
VoltChkPin Con AX0

index=0
filename="f001.log",0 ; next will be f002.log and so on. Null terminated
filenr=1
menumode=0
k=0
waitcount=0

bDeleteFile=0
bLaserOn=0
bChgFile=0
offset=0 ; mm 

enablehserial
sethserial H9600, H8DATABITS, HNOPARITY, H1STOPBITS

pause 2000
gosub LcdStart ; lcd startup screen
pause 5500 ; wait for sensors to startup
gosub SerialPortInit ; initialize serialports
;gosub ReadSetup ; read setup file
gosub LcdMeas ; 


; Setup external interrupt on P0 (WKP0)
enable WKPINT_0 ;enable the external interrupt
oninterrupt WKPINT_0, WKP0ISR ; where to go on interrupt
PMR5.BIT0=1 ; use P0 as WKP0
PUCR5.BIT0=1 ; turn on internal pull-up resistor
IEGR2.BIT0=0 ; 0 = high to low transition interrupt

; Setup external interrupt on P1 (WKP1)
enable WKPINT_1 ;enable the external interrupt
oninterrupt WKPINT_1, WKP1ISR ; where to go on interrupt
PMR5.BIT1=1 ; use P1 as WKP1
PUCR5.BIT1=1 ; turn on internal pull-up resistor
IEGR2.BIT1=0 ; 0 = high to low transition interrupt

; I2C bus control register
; ICCR.BIT7=1 ; use P6 and P7 as SDA,SCL (i2c) not I/O
; PCR5.BIT6=1 ; P6 an output
; PCR5.BIT7=0;  P7 an input
; problem med at P6 output er maks 3V, annan type utgang enn dei andre?

button0=0
button1=0
debounce0=0
debounce1=0

Main: ;    <------------------- Main loop starts here
  ;loop waiting for button pressed (external interrupt) button pressed measure, 1) compass, 2) laser 
  if (button0=1) then 
    if waitcount=6001 then ;start up from sleep in measure menu
      waitcount=0
      button0=0
      debounce0=0
      menumode=0 
      gosub LcdMeas
      goto Main
    endif
    waitcount=0 'reset timer
    pause 100; two button pressed within 50ms for double button
    if (button1=1) then
      Gosub DoubleButton
    else
      Gosub Button0Pressed
    endif  
    debounce0=0
  endif
  if (button1=1) then 
    if waitcount=6001 then ;start up from sleep in measure menu
      waitcount=0
      button1=0
      debounce1=0
      menumode=0 
      gosub LcdMeas
      goto Main
    endif
    waitcount=0 'reset timer
    pause 100; wait for double button, also 50ms dead button periode to avoid debounce   
    if (button0=1) then
      Gosub DoubleButton
    else
      Gosub Button1Pressed
    endif
    debounce1=0 ; now possible to press button1 again 
  endif
  pause 10
  if waitcount<=6000 then
    waitcount=waitcount+1 ; must count to 6001, if not always enter sleep procedure
  endif  
  if waitcount=6000 then  ' wait 60sec for sleep procedure
    ;turn off LCD and uMMC. save power
    gosub LcdCls
    ;high PwrOff
  endif 
goto Main ; <------------------- Loop ends here


;##################################
; BUTTON ROUTINES
;##################################

WKP0ISR: ; WKP0 interrupt service request
 disable WKPINT_0
 if (debounce0=0) then
   button0=1
   debounce0=1
 endif
 enable WKPINT_0
Resume

WKP1ISR: ; WKP1 interrupt service request
 disable WKPINT_1
 if (debounce1=0) then
   button1=1
   debounce1=1
 endif
 enable WKPINT_1
Resume

DoubleButton:
button0=0
button1=0
if menumode=0 then ; if in meas menu toggle laser spot on/off
  if bLaserOn=0 then ;if off then turn on
    serout TX1, I1200, ["s0o",CR,LF] ; commando for turn laser spot on
  else 
    serout TX1, I1200, ["s0p",CR,LF] ; commando for turn laser spot off
  endif  
  bLaserOn=bLaserOn+1
  if (bLaserOn=2) then 
    bLaserOn=0
  endif
endif
if menumode=1 then ;from undo -> advanced menu  
  menumode=4
  gosub LcdSendToPc
endif
if (menumode=6) and (bChgFile=1) then ; done chgFile
  bChgFile=0
  index=0 ; reset index
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Save + reset index"] 
  gosub WriteSetup
  gosub Button0Pressed
endif
debounce0=0
debounce1=0
return

Button0Pressed:
button0=0
bDeleteFile=0
if (bChgFile=1) and (menumode=6) then ; button0 is minus
  filenr=filenr-1
  if filenr=0 then 
    filenr=9
  endif 
  filename="f", dec filenr\3, ".log",0 
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Log file is ", str filename\8\0] 
else
  gosub ChgMenuMode
endif  
return
 
Button1Pressed:
button1=0
if menumode=0 then ; MEASURE
  gosub lcd1
  gosub Compass 
  gosub Laser
  if (laser_ok=1) and (compass_ok=1) then ;ok measurement
    index=index+1
    gosub Lcd2
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["                    "]
    gosub Lcd2
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Saving.."]
    ;lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$148] ; 9. pos 2. line
    ; disable buttons
    debounce0=1
    debounce1=1
    gosub SaveMeasure ;save result
    ;gosub WriteSetup ; save index
    gosub Lcd2
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode]  ", dec index\3, "   [Meas]"]
    if bLaserOn=1 then ; turn laser spot on again, it goes automatic off after a measure
      serout TX1, I1200, ["s0o",CR,LF] ; commando for turn laser spot on
    endif
    ; enable buttons
    debounce0=0
    debounce1=0
  endif  
endif
if (menumode=1) and (index>0) then ; first measurement is index 1, index=0 =>no measurment
  index=index-1
  gosub LcdUndo ;refresh
endif
if (menumode=2) then ; code for turn off
  ; code for shutdown
  ;gosub WriteSetup
  gosub PowerSave 
endif
; menumode=3 not existing, trick with barrier to Adv. menu
if menumode=4 then ; SEND TO PC
  if bDeleteFile=0 then
    gosub SendToPC ;first press button1
    gosub LcdDeleteFile
  endif  
  if bDeleteFile=1 then; Yes to delete if 1, if second time pressing button1, no if button0 pressed 
    gosub DeleteFile
    menumode=0
    gosub LcdMeas; goto measure menu
  endif  
  bDeleteFile=bDeleteFile+1 ; toogle if bDeleteFile=1 then Yes to delete file 
  if (bDeleteFile=2) then 
    bDeleteFile=0
  endif  
endif
if menumode=5 then ; LASER SPOT ON/OFF
  if bLaserOn=0 then ;if off then turn on
    serout TX1, I1200, ["s0o",CR,LF] ; commando for turn laser spot on
  endif
  if (bLaserOn=1) then
    serout TX1, I1200, ["s0p",CR,LF] ; commando for turn laser spot off
  endif  
  bLaserOn=bLaserOn+1
  if (bLaserOn=2) then 
    bLaserOn=0
  endif 
  gosub LcdLaserSpot
endif
if (menumode=6) then
  if (bChgFile=1) then
    filenr=filenr+1
    if filenr=10 then 
      filenr=1 
    endif
    filename="f", dec filenr\3, ".log",0 
    gosub Lcd1
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["File is ", str filename\8\0]
  endif  
  if (bChgFile=0) then
    bChgFile=1
    gosub Lcd1
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["File is ", str filename\8\0]
    gosub Lcd2
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[-]    [Done]    [+]"]
  endif
endif
if (menumode=7) then
  gosub ReadSetup
endif
if (menumode=8) then
  gosub WriteSetup
endif
return 

SerialPortInit:  
  hserout["S 1 20", CR]; and 20*10ms timeout after write/append
  gosub SDreply ; check for prompt >
  serout TX2, I2400, ["@F0.3=0*65",CR,LF] ;run compass in sample mode
  ;serin RX2, I2400, [str InData\20\LF] 
  ;(at startup maybe instr. read some Rs232 bytes)
  ;serout TX2, I2400, [CR,LF] ; empty command to Compass
  serout TX1, I1200, ["s0N02N",CR,LF] ; arbitrary command Laser (get serialnumber)  
  ;serin TX1, 500, jump5, [dummy] 
return

PowerSave:
  ; turn off LCD and sensors
  ;I2Cout SDA, SCL, $0,[($40), $0] ; controlbyte is ignored, address = device id+address(A0-A3)
  gosub lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["i"]
  pause 2000
  ;I2Cout SDA, SCL, $0, [($40), $ff]
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["c"]
  if 0=1 then
    jump8:
     lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["jump8"]
  endif
   high PwrOff ; turn off instrument
   pause 3000
   low PwrOff
return
  

; ###### Procedure for read True North Revolution 2X ###########
; ##   use RX2 (P5) and TX2 (P4) and 2400 baud                ##
; ##   Instr. must be in sample mode.                         ##
; ##   Ask for HTM line: $PTNT,HTM*63<cr><lf>                 ##
; ##   e.g. answer: $PTNTHTM,85.5,N,3.7,N,-2.6,N,73.2,3103*23 ##
; ##   is heading 85.5, pitch 3.7 (+ up). Roll -2.6 Dip 73.2  ##
; ##############################################################
Compass:
  compass_ok=0 ; assume not ok compass readout
  k=0
  do
    k=k+1
    serout TX2, I2400, ["$PTNT,HTM*63",CR,LF] ; ask for HTM sentence
    gosub Lcd1
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["H", dec k]
    pause 100
    n=0
    do
      serin RX2, I2400, jumpTo, 2000, jumpTo, [inCompass(n)]
      ;debug [" n",dec n,"#", inCompass(n)]
      n=n+1
    while  (inCompass(n-1) <> LF) and (n<43) ;k<43 virker. k<49 stopper ikkje p CR/LF
    ;debug ["bytes in htm=", dec n, CR, LF]
    if 0=1 then ; never enter except when timeout error. goto jumpTo
      jumpTo:
        lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Timeout"]
        pause 500
    endif
  while (inCompass(5)<>"H") and (k<3)  ; H is ascii 72
  if (inCompass(5)="H") then ; ok read
    compass_ok=1 ; ok compass readout
    strHeading = rep "X"\5 ; load "X" into array strHeading
    n=9
    while (inCompass(n)<>",") and (n<14)
      strHeading(n-9)= inCompass(n)
      n=n+1
    wend
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$181] ; position 2 first line
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [STR strHeading\5\"X"] ; write strHeading max 5 char or until X
    ;debug ["strHeading=", strHeading]
    n=n+3 ; start of pitch position
    m=0
    strTilt = rep "X"\5 ; load "X" into array strTilt
    while (inCompass(n)<>",") and (m<5) 
      strTilt(m)=inCompass(n)
      n=n+1
      m=m+1
    wend
    ; lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [" k=", dec k," "]
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [" ",STR strTilt\5\"X"]; write strTilt max 5 char or until X
  endif
  ; gosub Lcd2
  ; for n=0 to 19 ;  write whole string from compass
  ;   lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [inCompass(n)]
  ; next
return    
  
; #####   Procedure for read Dimitex. Find Distance    ###### 
; ##    use RX1 (P3) and TX1 (P2) and 1200 baud            ##
; ###########################################################
Laser:
  laser_ok=0  ; assume not ok laser measure         
  k=0
  do
    k=k+1
    InLaser = rep "X"\15 ; load 25 X to array
    serout TX1, I1200, ["s0g",CR,LF] ; commando for read distance
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$18D] ; set cursor to position 14 (start 1), firstline
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["D",dec k]
    ;pause 50 ; 100
    n=0
    do
      serin RX1, I1200, jump, 3000, jump, [inLaser(n)]  ; with 2000 got Timeout error always after first start up 
      n=n+1
    while ( inLaser(n-1) <> LF) and (n<12) ;shall receive 12 chars + CR 
    if (inLaser(0)="@") or (inLaser(2)="@") then ; error in reading
      lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$18D] ; set cursor to position 14 (start 1), firstline 
      for m=0 to n-1 
        lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [inLaser(m)]
      next 
      pause 2000
      lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$18D] ; set cursor to position 14 (start 1), firstline 
      ; if reply: @E203 => syntax error
      if (inLaser(2)="2") and (inLaser(3)="0") and (inLaser(4)="3") then
        lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["SyntaxE"]
      endif
      ; if reply: g0@E255   distance<250mm, too close
      if (inLaser(3)="E") and (inLaser(4)="2") and (inLaser(5)="5") and (inLaser(6)="5") then
        lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["D<250mm"]
      endif
      if 0=1 then ; only enter when timeout in serin
        jump:
         lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$18D] ; set cursor to position 14 (start 1), firstline
         lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Timeout"]
      endif
      pause 2000
    endif
  while (inLaser(3)<>"+") and (k<3) 
  if inLaser(3)="+" then ;ok measurement
    laser_ok=1 ; ok laser measure
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$18E] ;position 15 first line
    for n=5 to 10
      lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [inLaser(n)]
      strDistance(n-5)=inLaser(n)
    next 
    pause 2000
    ; calculate offset
    lDistance = str strDistance\6
    ;offset=14
    lDistance = lDistance+14
   ; lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$18E] ;position 15 first line
   ; lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [dec lDistance\6]
    ;gosub Lcd2  
    ;for n=0 to 11 ;write whole string from Dimetix
    ;lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [inLaser(n)]
    ;next 
  endif  
return

; ################### SD memory card routines ##########
; Write data to SD card
; use RX3 (P15/TXD) and TX3 (P14/RXD) and 9600 baud. 
; Only hardware serial ports (interrupt driven and 128 bytes buffer)
; ######################################################
SaveMeasure:
  k=0
  do 
    hserout ["O 2 A /", str filename\8\0, CR] ; open filehandle 1 for append with given filename
    gosub SDreply
    k=k+1
  while (SD_ok=0) and (k<3)
  l=6
  if SD_ok=0 then goto SaveError
  hserout ["W 2 24", CR] ; write to filehandle 1 next 24 bytes if not 24 bytes written, wait 200ms
  ;pause 100
  k=0
  do 
    hserout [dec index\3, 32, str strHeading\5\"X", 32, str strTilt\5\"X", 32, str strDistance\6, CR, LF]
    gosub SDreply
    k=k+1
  while (SD_ok=0) and (k<3)
  l=7
  if SD_ok=0 then goto SaveError
  k=0
  do 
    hserout ["C 2", CR] ; close filehandle 1
    gosub SDreply
    k=k+1
  while (SD_ok=0) and (k<3)
  l=8
  if SD_ok=0 then goto SaveError
  if 0=1 then
    SaveError:
      lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["E", dec l, " save"]
  endif
return

; SEND DATA FROM FILE TO SER_OUT (Console)
SendToPC:
  filesize=0
  filepos=0
  gosub lcd1
  k=0
  do 
    hserout ["O 1 R /", str filename\8\0, CR] ; open filehandle 1 for append with given filename
    gosub SDreply
    k=k+1
  while (SD_ok=0) and (k<3)
  do
    saveData= rep "X"\512
    hserout ["I 1", CR]
    hserin 1000, TError, [dec filepos]
    hserin 1000, TError, [dec filesize]
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$180] ; 1st line and home ($180)
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [dec filepos, "/", dec filesize,"B "]
    if filesize=0 then goto TError
    bytesToWrite = filesize - filepos
    ; nb! filepos and filesize are word 0..65500, maybe too narrow 
    if bytesToWrite>512 then 
      bytesToWrite=512
    endif  
    hserout ["R 1 ", dec bytesToWrite, " ", dec filepos, CR]; read file 1, all bytes but maks 512 bytes. from pos 0
    ;lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$18D] ; set cursor to position 14 (start 1), firstline
    ;lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [dec filepos]
    ;gosub SDreply klikker og skriv masse til displayet, fr med ein space frst n
    n=0
    hserin 2000, TError, [dummy] ; read the first space that are always after Read File
    do 
      hserin 2000, TError, [saveData(n)]
      if saveData(n)<>">" then 
        serout S_OUT, I9600, [saveData(n)]
      endif  
      n=n+1
    while (saveData(n-1)<>">") and (n<=512)  
    bytesToWrite=filesize-filepos
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$180] ; 1st line and home ($180)
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [dec filepos+bytesToWrite, "/", dec filesize,"B "]
    ;lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [dec bytesToWrite, "B"]
  while (bytesToWrite>512)  
  TError:
   hserout ["C 1", CR]
   lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["End"]
   pause 2000
return

; Delete data file and reset index to 0. Update setup.txt
DeleteFile:
  hserout ["E /", str filename\8\0, CR] ;erase savefile
  gosub SDreply
  index=0
  ;gosub WriteSetup
return

; READ SETUP FILE FOR FILENAME AND INDEX VALUE
ReadSetup:
  k=0
  gosub Lcd1
  do
    ; read from setup file last index and filename
    hserout ["O 1 R /setup.txt", CR] ; open filehandle 1 for append with given filename
    gosub SDreply
    k=k+1
  while (sd_ok=0) and (k<3)
  l=0
  if sd_ok=0 then 
    ; if setup.txt does not exist create it
    pause 3000
    gosub WriteSetup 
    hserout ["O 1 R /setup.txt", CR] 
    gosub SDreply
    if sd_ok=0 then goto ReadError ; still problem
  endif  
  l=1
  k=0 
  hserout ["R 1 14", CR]; read file 1, 14 bytes from pos 0(8 bytes filename always 8 byte e.g. f001.log, f002.log
  hserin 3000, ReadError, [str InLaser\2\"f"] ;space and f
  hserin 3000, ReadError, [dec filenr] ;number part
  hserin 3000, ReadError, [str InCompass\6\LF] ; 1.line; f001.log<cr><lf> so read max 10 bytes or until LF
  l=2
  hserin 3000, ReadError, [dec index] ; read number until a unrecognized byte(Q), then convert to number 
  ; in hserin dec converts ascii to int, in hserout int to ascii
  l=3
  hserin 1000, ReadError, [dummy] ; read the > prompt reply from uMMC
  k=0
  do 
    hserout ["C 1", CR] ; close filehandle 1
    gosub SDreply
    k=k+1
  while (sd_ok=0) and (k<3)
  l=4
  if sd_ok=0 then goto ReadError
  
  filename="f", dec filenr\3,".log",0
  ;for n=1 to 8 ; n=0 is space. See uMMC manual
  ;  filename(n-1)=InLaser(n)
  ;next
  ;filename(8)=0 ; null terminated
  if (filename(0)<>"f") and (filename(4)<>".") then ;something wrong in reading
    filename="f001.log",0
    filenr=1
  endif
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["R", STR filename\8\0, " ", dec index\3]
  if 0=1 then ;only jump to on error
    ReadError:
      ;gosub Lcd1
      if l=0 then ;error open setup.file
       lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["E open setup"] 
      endif  
      if l>0 then
       lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["E", dec l," read setup"]
       hserout ["C 1", CR] ; close filehandle 1 
      endif
  endif    
return

; # SAVE FILENAME AND INDEX TO SETUP FILE
WriteSetup:
  ;gosub lcd1
  hserout ["E /setup.txt", CR] ;erase file, not possible to write over existing file. Give error code EF4
  gosub SDreply
  ;lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [STR filename\8\0, dec index\3] 
  hserout ["O 1 W /setup.txt", CR] ; open filehandle 1 for append with given filename
  gosub SDreply
  hserout ["W 1 14", CR] ; write to filehandle 1 next 14 bytes (timeout set to 200ms) 8+ 2 + 2 +1
  hserout [STR filename\8\0, CR, LF, dec index\3, "Q"] ; mark file end with Q
  gosub SDreply
  hserout ["C 1", CR] ; close filehandle 1
return

; READ REPLY FROM uMMC ON COMMANDO, OK WHEN FIRST BYTE > or SPACE
SDreply:
  InData(0)="X"
  n=0
  do
    hserin 1000, Jump3, [InData(n)]
    n=n+1
  while (InData(n-1)<>">") or (InData(n-1)<>" ")
  Jump3:
  if (InData(0)=">") or (InData(0)=" ") then
    SD_ok=1
  else
    SD_ok=0
  endif 
  if (InData(0)="E") and (InData(1)="F") and (InData(2)="1") then
    hserout ["C 1", CR] ; EF1> error = handle in use, so close filehandle 1
  endif
  for m=0 to n-1
    lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [InData(m)]
  next   
return

ChkVoltage:
  ADIN VoltChkPin, ADinput
  BatteryVolt=8.69*(5.04/1023)*ToFloat(ADinput) ; 8.69 due to 130k and 1M voltage divider 
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Battery=",real BatteryVolt\4,"V "]
return

; ################## ERROR HANDLING ######################
; # Timeout after serial port reading
; # Parity error if wrong parity received
; ########################################################

TimeoutError:
lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4,[" ETimeout"]
pause 1000
return

ParityError:
lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4,[" EParity"]
pause 1000
return

; ################# LCD ROUTINES #######################
; # Menu controlled by external interrupt (P0 & P1)  
; ######################################################
ChgMenuMode: ; button0 pressed (red)
  menumode=menumode+1
  if (menumode=3) or (menumode>6) then 
    menumode=0 
  endif
  if (menumode=0) then 
    gosub LcdMeas
  endif
  if (menumode=1) then
    gosub LcdUndo
  endif
  if (menumode=2) then
    gosub LcdTurnOff
  endif
  ; next are part of advanced menu
  if (menumode=4) then
    gosub LcdSendToPC
  endif 
  if (menumode=5) then
    gosub LcdLaserSpot
  endif  
  if (menumode=6) then
    gosub LcdChgFile
  endif
  if (menumode=7) then
    gosub LcdReadSetup
  endif
  if (menumode=8) then
    gosub LcdWriteSetup
  endif 
return

;LCD init + startup screen
LcdStart:
  lcdinit RegSel\Clk\DB7\DB6\DB5\DB4, RdWrPin  ; Initialize LCD 
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$101,$10E] ;LCD clear and no blinking cursor
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["CaveMap v0.1"]
return

;measurement screen
LcdMeas:
  gosub LcdCls
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode]   ", dec index\3, "  [Meas]" ]
  gosub Lcd1
return

LcdUndo
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Undo index ", dec index\3, "?"] 
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Mode  Adv.Meny  Undo"]
return
 
LcdTurnOff
  gosub Lcd1
  gosub ChkVoltage 'display battery voltage for 2sec
  sleep 2000
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Turn off CaveMap?"] 
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode]         [Off]"]
return

;next screens are from advanced menu 
LcdSendToPC:
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [str filename\4\0, " -> PC"] 
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode2]       [Send]"]
return
 
LcdLaserSpot:
  gosub Lcd1
  if bLaserOn=0 then
    inData="o","n",0
  else
    inData="o","f","f",0  
  endif  
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Laser spot ", str inData\3\0, "?"] 
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode2]         [Do]"]
return 

LcdChgFile:
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["File is ", str filename\8\0] 
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode2]     [Change]"]
return

LcdReadSetup:
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["ReadSetup?"]
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode2]       [Read]"]
return

LcdWriteSetup:
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Writesetup"]
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode2]      [Write]" ]
return
;next not used
LcdSaveData:
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Save measurement?"]
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[Mode]          [Do]"]
return

LcdDeleteFile: 
  gosub Lcd1
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["Delete ", str filename\8\0, "?"] 
  gosub Lcd2
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["[No]           [Yes]"]
return
; end not used

; clear lcd display, cursor at 1.line, 1.position
LcdCls:
 lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$101]
return

; clear 1. line and move cursor to 1. line, 1. position of display
Lcd1:
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$180] ; 1st line and home ($180)
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, ["                    "] ; blank 1. line
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$180]
return

; move cursor to 2. line of display
Lcd2:
  lcdwrite RegSel\Clk\DB7\DB6\DB5\DB4, [$1C0] ; go to 2. line ($1C0) and 1. postion
return

end ; end of Program
